#!/usr/bin/python

import commands
import os
import pwd
import random
import re
import sys

INDENT = 2

def give_usage(exit_code = 1):
    os.system('less ' + os.path.dirname(__file__) + '/./lib/usage')
    sys.exit(exit_code)

if os.getenv('QUERY_STRING'):
    should_cache = False
else:
    should_cache = True
username = pwd.getpwuid(os.getuid())[0]

if should_cache:
    try:
        verse = open('/tmp/bibleverse').read()
    except:
        verse = None
    forked = not os.fork()
else:
    verse = None
    forked = False
if forked or not verse or not should_cache:
    if forked:
        sys.stdout = open('/tmp/bibleverse' + '.' +
          str(os.getpid()), 'w')
    all_verses = open(os.path.dirname(__file__) + '/./lib/versions/' +
      os.getenv('BIBLE_VERSION', 'KJV')).read().split('\n')
    patterns = open(os.path.dirname(__file__) + '/./lib/patterns').read().split('\n')

    if len(sys.argv) > 1:
        arguments = ' '.join(sys.argv[1:])
    else:
        arguments = ''
    if (arguments in ('--help', '--usage')):
        give_usage(0)
    pattern = ''
    for candidate in patterns:
        if not pattern:
            broken_down = re.search(r'^/([^/]*)/(\w*):/([^/]*)/(\w*)', candidate)
            if broken_down:
                argument_pattern = broken_down.group(1)
                argument_modifiers = broken_down.group(2)
                text_pattern = broken_down.group(3)
                text_modifiers = broken_down.group(4)
                if re.match(argument_pattern, arguments, re.IGNORECASE):
                    pattern = text_pattern

    selected_verses = []

    for verse in all_verses:
        if re.match(pattern, verse, re.IGNORECASE):
            selected_verses.append(verse)

    verse = selected_verses[random.randrange(len(selected_verses))]

    workbench = ''

    first_colon = False
    second_colon = False
    for character in verse:
        if first_colon and not second_colon and character == ' ':
            workbench += ':'
            second_colon = True
        if character == ':':
            if first_colon:
                second_colon = True
            first_colon = True
        workbench += character

    try:
        columns = commands.getoutput('tput cols')
        columns = int(re.sub(r'\s', '', columns))
    except:
        columns = 80

    remaining = workbench
    pieces = []
    while remaining:
        if len(remaining) < columns:
            pieces.append(remaining)
            remaining = None
        else:
            index = columns - 1
            while index > 1 and remaining[index] != ' ':
                index -= 1
            pieces.append(remaining[:index])
            remaining = ' ' * INDENT + remaining[index + 1:]

    print ''
    for piece in pieces:
        print piece
    print ''
    if forked:
        sys.stdout.close()
        os.rename('/tmp/bibleverse' + '.' + str(os.getpid()),
          '/tmp/bibleverse' )
